/* bwMgmt.c - initialize bandwidth mgmt module */

/* Copyright (c) 2007, TeamF1, Inc. */

/*
modification history
--------------------
01g,18aug11,swp changes for memory leak
01f,30Jul11,swp changes for spr 24777
01e,1July10,ymi merged changes for SPR 18574
01d,17nov08,sam changes under BWMGMT_UTM
01c,31mar07,rnp added check for rate is 0
01b,12Mar07,ayz added support for sfq qdisc  dynamically adding 
01a,28Feb07,rnp created
*/

/*
DESCRIPTION

This module implements the bandwidth management module

*/

#include "linux/module.h"
#include <linux/kernel.h>
#include <linux/version.h>
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,18)
#include <linux/config.h>
#endif /* LINUX_VERSION_CODE */
#include <linux/param.h>
#include <linux/rtnetlink.h>
#include <linux/pkt_sched.h>
#include <linux/netfilter_ipv4/ipt_BWPROF.h>

/* parent is always 1:1 */
#define BWMGMT_UTM

#ifdef BWMGMT_UTM
/* changing the parent to 1:2 for UTM */
#define BWMGMT_PARENT_CLASS (2)
#else
/* parent is always 1:1 */
#define BWMGMT_PARENT_CLASS (1)
#endif

#define get_hz()    (HZ)
#define NLMSG_TAIL(nmsg) \
	((struct rtattr *) (((void *) (nmsg)) + NLMSG_ALIGN((nmsg)->nlmsg_len)))

/* module params */
static unsigned int phandle = 1;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,25)
module_param(phandle, uint, 0);
#else /* LINUX_VERSION_CODE */
MODULE_PARM(phandle, "i");
#endif /* LINUX_VERSION_CODE */
MODULE_PARM_DESC(phandle, "class handle");
extern int tc_modify_qdisc_ex(struct sk_buff *skb, struct nlmsghdr *n, void *arg);
extern int tc_ctl_tclass_ex(struct sk_buff *skb, struct nlmsghdr *n, void *arg);
static double tick_in_usec = 1;

/* from tc */
static int addattr_l(struct nlmsghdr *n, int maxlen, int type, const void *data,
	      int alen)
    {
    int len = RTA_LENGTH(alen);
    struct rtattr *rta;

    if (NLMSG_ALIGN(n->nlmsg_len) + RTA_ALIGN(len) > maxlen) {
        printk(KERN_ERR "addattr_l ERROR: message exceeded bound of %d\n",
               maxlen);
        return -1;
    }
    rta = NLMSG_TAIL(n);
    rta->rta_type = type;
    rta->rta_len = len;
    memcpy(RTA_DATA(rta), data, alen);
    n->nlmsg_len = NLMSG_ALIGN(n->nlmsg_len) + RTA_ALIGN(len);
   
    return 0;
    }

/* from tc_core.c */
long tc_core_usec2tick(long usec)
    {
    return usec*tick_in_usec;
    }

long tc_core_tick2usec(long tick)
    {
    return tick/tick_in_usec;
    }

unsigned tc_calc_xmittime(unsigned rate, unsigned size)
    {
    return tc_core_usec2tick(1000000*((double)size/rate));
    }

int tc_calc_rtable(unsigned bps, __u32 *rtab, int cell_log, unsigned mtu,
		   unsigned mpu)
    {
    int i;
    unsigned overhead = (mpu >> 8) & 0xFF;
    mpu = mpu & 0xFF;

    if (mtu == 0)
        mtu = 2047;

    if (cell_log < 0) {
        cell_log = 0;
        while ((mtu>>cell_log) > 255)
            cell_log++;
    }
    for (i=0; i<256; i++) {
        unsigned sz = (i<<cell_log);
        if (overhead)
            sz += overhead;
        if (sz < mpu)
            sz = mpu;
        rtab[i] = tc_core_usec2tick(1000000*((double)sz/bps));
    }
    return cell_log;
    }

int tc_core_init(void)
{
    extern int psched_tick_per_us, psched_us_per_tick;
    tick_in_usec = (double)psched_tick_per_us/psched_us_per_tick;
    return 0;
}

/* End - from tc_core.c */

/* 
 * this routine is based on htb_parse_class_opt() and tc_class_modify() in tc 
 * It adds an HTB class 1:<classid>
 */
int bwClassCtl
    (
    uint32_t classid,
    uint32_t cmd,
    uint32_t ifindex,
    uint32_t rate,
    uint32_t ceil,
    uint32_t prio
    )
    {
    struct {
        struct nlmsghdr 	n;
        struct tcmsg 		t;
        char   			buf[4096];
    } * req = NULL;
    struct rtattr *tail, *tca[TCA_OPTIONS];
    struct tc_htb_opt opt;
    __u32 rtab[256], ctab[256];
    unsigned buffer=0,cbuffer=0;
    int cell_log=-1,ccell_log = -1;
    unsigned mtu, mpu;
    unsigned char mpu8 = 0, overhead = 0;
    int flags = NLM_F_EXCL|NLM_F_CREATE;
    char  k[16] = "htb";
    int ret = 0;

    req = kmalloc(sizeof(*req), GFP_ATOMIC);
    if (req == NULL)
        {
        ret = -ENOMEM;
        goto bwClassCtlError;
        }
    memset(req, 0, sizeof(*req));
    req->n.nlmsg_len = NLMSG_LENGTH(sizeof(struct tcmsg));
    req->n.nlmsg_flags = NLM_F_REQUEST|flags;
    req->n.nlmsg_type = cmd;
    req->t.tcm_family = AF_UNSPEC;
    req->t.tcm_handle = TC_H_MAKE(phandle,classid);
    req->t.tcm_parent = TC_H_MAKE(phandle,BWMGMT_PARENT_CLASS);
    req->t.tcm_ifindex = ifindex;

    memset(&opt, 0, sizeof(opt)); 
    mtu = 1600; /* eth packet len */

    opt.rate.rate = rate * (1000/8); /* convert to bytes-per-sec */
    opt.ceil.rate = ceil * (1000/8); /* convert to bytes-per-sec */

    if (!opt.rate.rate)
        opt.rate.rate = 1; /* min is 1 byte-per-sec */

    /* if ceil params are missing, use the same as rate */
    if (!opt.ceil.rate) 
        opt.ceil = opt.rate;
    /* Setting prio. It should be within the range 0-2. But adding
     * checks anyway. */
    if (prio < 0)
        opt.prio = 0;
    else if (prio > 2)
        opt.prio = 2;
    else
        opt.prio = prio;

    /* compute minimal allowed burst from rate; mtu is added here to make
       sute that buffer is larger than mtu and to have some safeguard space */
    if (!buffer) 
        buffer = opt.rate.rate / get_hz() + mtu;
    if (!cbuffer) 
        cbuffer = opt.ceil.rate / get_hz() + mtu;

    /* encode overhead and mpu, 8 bits each, into lower 16 bits */
    mpu = (unsigned)mpu8 | (unsigned)overhead << 8;
    opt.ceil.mpu = mpu; opt.rate.mpu = mpu;

    if ((cell_log = tc_calc_rtable(opt.rate.rate, &rtab, cell_log, mtu, mpu)) < 0) {
        printk(KERN_ERR "htb: failed to calculate rate table.\n");
        ret = -1;
        goto bwClassCtlError;
    }
    opt.buffer = tc_calc_xmittime(opt.rate.rate, buffer);
    opt.rate.cell_log = cell_log;

    if ((ccell_log = tc_calc_rtable(opt.ceil.rate, &ctab, cell_log, mtu, mpu)) < 0) {
        printk(KERN_ERR "htb: failed to calculate ceil rate table.\n");
        ret = -1;
        goto bwClassCtlError;
    }
    opt.cbuffer = tc_calc_xmittime(opt.ceil.rate, cbuffer);
    opt.ceil.cell_log = ccell_log;
    addattr_l(&req->n, sizeof(*req), TCA_KIND, k, strlen(k)+1);
    tail = NLMSG_TAIL(&req->n);
   
    addattr_l(&req->n, sizeof(*req), TCA_OPTIONS, NULL, 0);
    addattr_l(&req->n, sizeof(*req), TCA_HTB_PARMS, &opt, sizeof(opt));
    addattr_l(&req->n, sizeof(*req), TCA_HTB_RTAB, &rtab, sizeof(__u32)*256);
    addattr_l(&req->n, sizeof(*req), TCA_HTB_CTAB, &ctab, sizeof(__u32)*256);
    tail->rta_len = (void *) NLMSG_TAIL(&req->n) - (void *) tail;
    memset(tca, 0, sizeof(tca));
    tca[TCA_OPTIONS-1]=tail;
    ret = tc_ctl_tclass_ex(NULL, &req->n, tca);

    bwClassCtlError:
        {
        if (req) kfree(req);
        return ret;
        }
    }

#ifdef BWMGMT_UTM
int bwQdiscCtlLeafAdd
    (
    uint32_t    cmd, 
    uint32_t    parent,
    uint32_t    ifindex,    
    int         leafnum
    )
    {
    int ret = -1;
    struct rtattr *tail, *tca[__TCA_MAX];
    int    handle=0;
    struct tc_sfq_qopt opt;
    char  k[16] = "sfq";
    struct {
        struct nlmsghdr 	n;
        struct tcmsg 		t;
        char   			buf[256];
    } *req;
    req=NULL;
    req = kmalloc(sizeof(*req), GFP_ATOMIC);

    if (req == NULL)
        {
        ret = -ENOMEM;
        goto bwQdiscCtlError;
        }
    handle = parent * 10 + leafnum;
    memset(req, 0, sizeof(*req));
    req->n.nlmsg_len = NLMSG_LENGTH(sizeof(struct tcmsg));
    req->n.nlmsg_flags = NLM_F_EXCL|NLM_F_CREATE|NLM_F_REQUEST|NLM_F_ACK;
    req->n.nlmsg_type = cmd;
    req->t.tcm_family = AF_UNSPEC;
    req->t.tcm_handle =TC_H_MAKE(handle << 16 ,0);
    req->t.tcm_parent = TC_H_MAKE(parent << 16 ,leafnum);
    req->t.tcm_ifindex = ifindex;

    memset(tca,0x0,sizeof(tca));
    memset(&opt,0x0,sizeof(opt));

    opt.perturb_period=10;

    tca[TCA_KIND-1]= NLMSG_TAIL(&req->n);
    addattr_l(&req->n, sizeof(*req), TCA_KIND, k, strlen(k)+1);
    tail = NLMSG_TAIL(&req->n);

    addattr_l(&req->n, sizeof(*req), TCA_OPTIONS,&opt ,sizeof(opt));
    tail->rta_len = (void *) NLMSG_TAIL(&req->n) - (void *) tail;
    tca[TCA_OPTIONS-1]=tail;

    ret= tc_modify_qdisc_ex(NULL, &req->n, tca);        
    bwQdiscCtlError:
        if (req) kfree(req);
        return ret;    
    }
#endif /* BWMGMT_UTM */

/* 
 * this routine is based on sfq_parse_class_opt() and tc_qdisc_modify() in tc 
 * It adds an SFQ qdisc under each class 1:<parent>
 */
int bwQdiscCtl
    (
    uint32_t cmd, 
    uint32_t parent,
    uint32_t ifindex
    )
    {
    int ret = -1;
    struct rtattr *tail, *tca[__TCA_MAX];
#ifndef BWMGMT_UTM        
    struct tc_sfq_qopt opt;
    char  k[16] = "sfq";
#else /* BWMGMT_UTM */
    struct tc_prio_qopt opt={5, 0, { 1, 2, 2, 2, 1, 2, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1 }, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0,0,0,0}};

    char  k[16] = "prio";
#endif /* BWMGMT_UTM */   
    struct {
        struct nlmsghdr 	n;
        struct tcmsg 		t;
        char   			buf[256];
    } *req;
    req=NULL;
    req = kmalloc(sizeof(*req), GFP_ATOMIC);

    if (req == NULL)
        {
        ret = -ENOMEM;
        goto bwQdiscCtlError;
        }

    memset(req, 0, sizeof(*req));
    req->n.nlmsg_len = NLMSG_LENGTH(sizeof(struct tcmsg));
    req->n.nlmsg_flags = NLM_F_EXCL|NLM_F_CREATE|NLM_F_REQUEST|NLM_F_ACK;
    req->n.nlmsg_type = cmd;
    req->t.tcm_family = AF_UNSPEC;
    req->t.tcm_handle =TC_H_MAKE(parent << 16 ,0);
    req->t.tcm_parent = TC_H_MAKE(phandle << 16 ,parent);
    req->t.tcm_ifindex = ifindex;

    memset(tca,0x0,sizeof(tca));
#ifndef BWMGMT_UTM            
    memset(&opt,0x0,sizeof(opt));
    opt.perturb_period=10;
#endif /* BWMGMT_UTM */

    tca[TCA_KIND-1]= NLMSG_TAIL(&req->n);
    addattr_l(&req->n, sizeof(*req), TCA_KIND, k, strlen(k)+1);
    tail = NLMSG_TAIL(&req->n);
    addattr_l(&req->n, sizeof(*req), TCA_OPTIONS,&opt ,sizeof(opt));
    tail->rta_len = (void *) NLMSG_TAIL(&req->n) - (void *) tail;
    tca[TCA_OPTIONS-1]=tail;

    ret= tc_modify_qdisc_ex(NULL, &req->n, tca);
    if (ret < 0) goto bwQdiscCtlError;

#ifdef BWMGMT_UTM    
    /* Add leaf nodes to the prio qdisc added above */
    ret = bwQdiscCtlLeafAdd(RTM_NEWQDISC, parent, ifindex, 1);
    if (ret < 0) goto bwQdiscCtlError;

    ret = bwQdiscCtlLeafAdd(RTM_NEWQDISC, parent, ifindex, 2);
    if (ret < 0) goto bwQdiscCtlError;

    ret = bwQdiscCtlLeafAdd(RTM_NEWQDISC, parent, ifindex, 3);
    if (ret < 0) goto bwQdiscCtlError;

    ret = bwQdiscCtlLeafAdd(RTM_NEWQDISC, parent, ifindex, 4);
    if (ret < 0) goto bwQdiscCtlError;

    ret = bwQdiscCtlLeafAdd(RTM_NEWQDISC, parent, ifindex, 5);
    if (ret < 0) goto bwQdiscCtlError;
#endif /* !BWMGMT_UTM */

    bwQdiscCtlError:
        if (req) kfree(req);
        return ret;
    }

int bwProfInit(void)
    {
    extern BW_MGMT bwMgmt;
    memset(&bwMgmt, 0, sizeof(bwMgmt));
    bwMgmt.classCtl = bwClassCtl;
    bwMgmt.qdiscCtl = bwQdiscCtl;
    
    return 0;
    }

int bwProfExit(void)
    {
    extern BW_MGMT bwMgmt;
    memset(&bwMgmt, 0, sizeof(bwMgmt));
    return 0;
    }

/*******************************************************************************
*
* bwMgmtInit - initialize bw mgmt module
*
* This routine initializes bw mgmt module
*
* RETURNS: 0 or -1
*/
int __init bwMgmtInit (void)
    {
    tc_core_init();
    bwProfInit();
    return (0);
    }

/*******************************************************************************
*
* bwMgmtExit - de-init bw mgmt module
*
* This routine deinitializes bw mgmt module
* 
* RETURNS: NONE
*/
void __exit bwMgmtExit (void)
    {
    bwProfExit();
    }

module_init(bwMgmtInit);
module_exit(bwMgmtExit);
