/* igmpConfig.c - igmp config utility */

/* Copyright (c) 2008, TeamF1, Inc. */

/*
 * modification history
 * --------------------
 * 01d,20sep08,chd redirecting to output of adpCmdExec to /dev/null, to avoid
 *                 console errors.
 * 01c,11aug08, vpp redirecting error to /dev/null.
 * 01b,10jun08, anp cleaned up.
 * 01a,06mar08, sau written.
 */

/*
DESCRIPTION
This file is the the linux entry point for management routines which calls the
IGMP binary.

INCLUDE FILES: adaptos.h umi.h
*/

/* includes */
 
#include <adaptos.h>
#include <umi.h>

/* defines */

#define KILL_IGMP      "/usr/bin/killall -9 igmpproxy"

/* locals */

ADP_MOD_ID(UMI_COMP_IGMP);

#include "../../mgmt/igmpMgmt.c"

/*******************************************************************************
*
* main - management entry point for linux IGMP backend.
*
* This routine is the main for IGMP management backend.
* 
* RETURNS: OK on success, ERROR otherwise.
*
* ERRNO: N/A
*
* SEE_ALSO: N/A
*/

int main 
    (
    int     argc,  /* number of arguments */
    char ** ppArgv /* argument vectors    */
    )
    {
    char * pProgName    = NULL;
    int    offsetToHash = 0;

    /* search for the command corresponding to this program name */
    pProgName = ppArgv [0];
    while (*pProgName)
        {
        if (*pProgName == '/')
            offsetToHash = (pProgName - ppArgv [0]) + 1;
        pProgName++;
        }
    pProgName = ppArgv [0] + offsetToHash;

    /* for ADP_DEBUG macros */
    umiInit(); 

    if (strcmp (pProgName, "igmpConfig") == 0)
        {
        if (argc >= 5)
            {
            igmpMgmtInit (ppArgv [1]);
            return (igmpMgmtDBUpdateHandler (ppArgv [1], atoi (ppArgv [2]),
                                             ppArgv [3], atoi (ppArgv [4]),
                                             &ppArgv [5], argc - 5));
            }

        ADP_PRINTF ("Usage:%s <DBFile> <opType> <tblName> <rowId>\n", 
                    ppArgv [0]);
        }
    else
        ADP_PRINTF ("Illegal invocation of igmpConfig (%s)", pProgName);

    return (ERROR);
    }
