--[[
#### Abhinendra Babu. T
#### TeamF1
#### www.TeamF1.com
#### June 07, 2008
#### File: bwLimit.lua
#### Description: Bandwidth Profiles and Traffic Selector Functions

#### Revisions: Venu Babu.M
#### Modified AUG 25,2011
#### Description: functions bwProfiles.profileStatusConfig - Added
-- Modification History
-- 27Jun12,sig  Fixed SPR 32877
]]--

--************* Requires *************

--************* Initial Code *************

--package bwProfiles
bwProfiles = {}
trafficSelector = {}

--************* Functions *************
-- Bandwidth Profiles Configuration
function bwProfiles.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end

-- Bandwidth Profiles Configuration
function bwProfiles.profileConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    -- For Add Operation
    if (operation == "add") then
	            valid = bwProfiles.config(inputTable, rowid, "add")
    end
    
    -- For Edit Operation
    if (operation == "edit") then
        if (DBTable == "BandWidthProfile") then
            local bwProfileName = db.getAttribute("BandWidthProfile", "_ROWID_", rowid, "ProfileName")
            local rows = db.getRows("TrafficSelector", "ProfileName", bwProfileName)
            local fwwhere = "BandwidthProfileName = '"..bwProfileName.."' AND Status = '1'"
            local fwrows = db.getRowsWhere("FirewallRules", fwwhere)    
            if (#rows ~= 0) then
               	db.rollback()
                return "ERROR", "BWPROFILE_INUSE_EDIT_FAILED"
           elseif(#fwrows ~= 0) then
                db.rollback()
                return "ERROR", "BWPROFILE_FW_INUSE_EDIT_FAILED"
            else
                valid = bwProfiles.config(inputTable, rowid, operation)            	
            end
        else
            valid = bwProfiles.config(inputTable, rowid, operation)            
        end
    end
    
     -- return
    if (valid) then
    	db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
    	db.rollback()	    
	return "ERROR", "BWPROFILE_CONFIG_FAILED"
    end
    	    
end

-- Bandwidth Profiles Deletion
function bwProfiles.deleteProfiles (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local case = ""
    
    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["BandWidthProfile._ROWID_"] = v;
    	local dscpMark = db.getAttribute("BandWidthProfile", "_ROWID_", v, "DscpMark")
        local bwProfileName = db.getAttribute("BandWidthProfile", "_ROWID_", v, "ProfileName")
    	local rows = db.getRows("TrafficSelector", "ProfileName", bwProfileName)
        local fwrows = db.getRows("FirewallRules", "BandwidthProfileName", bwProfileName)
    	local dscpTable = {}
	    dscpTable["DscpClassList.Status"] = 0
        if (#rows ~= 0) then
	        valid = false
	        case = "1"
        elseif(#fwrows ~=0) then
                valid = false
                case ="2"               
    	else 
	        valid = true
    	end
	    if (valid) then
	        valid = db.delete("BandWidthProfile", inTable)
    	end
	    if (not valid) then
	        break
    	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        if (case == "1") then
   	    return "ERROR", "BWPROFILE_USE_DELETE_FAILED"
        elseif(case == "2") then
            return "ERROR", "BWPROFILE_FW_USE_DELETE_FAILED"
	else
	    return "ERROR", "BWPROFILE_DELETE_FAILED"
	end
    end
end

-- Traffic Selector Configuration
function trafficSelector.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end

-- Traffic Selector Configuration
function trafficSelector.profileConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    valid = trafficSelector.config(inputTable, rowid, operation)
       	
    -- return
    if (valid) then
    	db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
    	db.rollback()
        if (operation == "delete") then
            return "ERROR", "TRAF_SELECTOR_DELETE_FAILED"            
        else
        	return "ERROR", "TRAF_SELECTOR_CONFIG_FAILED"            
        end
    end
end

-- Bandwidth Profile Status Configuration
function bwProfiles.profileStatusConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false            
    -- For Edit Operation
    local bwProfileStatus = {}
    bwProfileStatus["BandWidthProfileStatus.Status"] = inputTable["BandWidthProfileStatus.Status"]    
    valid = bwProfiles.config(bwProfileStatus, "1", "edit")
    
     -- return
    if (valid) then
    	db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
    	db.rollback()
	return "ERROR", "BWPROFILESTATUS_CONFIG_FAILED"
    end
end
