-- ddnsCLI.lua 
-- Bhanu R
-- TeamF1
-- 09/19/08
--
-- Modification History
-- 19sep08 rbh written
--
-- Description
-- CLISH DDNS LUA routines
--

require "teamf1lualib/ddns"
--
--
function ddnsGet (args)
    local currentWanStatus = ""
    local currentWan = ""
	local wanModeConfigRow = db.getRow("WanMode", "_ROWID_", "1")
	if (wanModeConfigRow["WanMode.Wanmode"] == "0") then
		currentWanStatus = "Single Port " .. wanModeConfigRow["WanMode.DedicatedLogicalIfName"] or ''
		currentWan = wanModeConfigRow["WanMode.DedicatedLogicalIfName"] or ''
	elseif (wanModeConfigRow["WanMode.Wanmode"] == "2") then
		currentWanStatus = "Auto Rollover with Primary " .. wanModeConfigRow["WanMode.FailoverPriLogicalIfName"] or ''
		currentWan = wanModeConfigRow["WanMode.FailoverPriLogicalIfName"] or ''
	else
		currentWanStatus = "Load Balancing"
		currentWan = "ALL"
	end
    printLabel ("WAN Mode")
    print("\n" .. currentWanStatus .. "\n")
    for i = 1, 4 do
        local resultTab = {}
        local ddnsTable = nil

        local ddnsService = db.getAttribute ("ddns", "_ROWID_", i , "ddnsService")
        if ((currentWan ~= "" and currentWan ~= "ALL" and currentWan ~= "WAN" .. i) or (ddnsService == "0")) then
            printLabel ("WAN" .. i .. " Dynamic DNS service currently disabled")
        else
            printLabel ("WAN" .. i .. " Dynamic DNS Service currently enabled.")
            ddnsTable = db.getRow ("ddns", "_ROWID_", i)
            resTab.insertField (resultTab, "Host Name ", ddnsTable["ddns.hostname"] or "")
            resTab.insertField (resultTab, "Username ", ddnsTable["ddns.username"] or "")
            if (ddnsTable["ddns.timePeriod"] == "1") then
                resTab.insertField (resultTab, "Time Period ", "Enabled")
            else
                resTab.insertField (resultTab, "Time Period ", "Disabled")
            end
            if (ddnsTable["ddns.wildflag"] == "1") then
                resTab.insertField (resultTab, "Wild Flag ", "Enabled")
            else
                resTab.insertField (resultTab, "Wild Flag ", "Disabled")
            end

            resTab.print (resultTab, 0)
        end
    end
end

--
-- This routine saves the modified LAN configuration information.
function ddnsCfgSave(configRow)	 
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "ddns"

    local currentWan = ""
    local wanModeRow = db.getRow("WanMode", "_ROWID_", "1")
	if (wanModeRow["WanMode.Wanmode"] == "0") then
		currentWan = wanModeRow["WanMode.DedicatedLogicalIfName"] or ''
	elseif (wanModeRow["WanMode.Wanmode"] == "2") then
		currentWan = wanModeRow["WanMode.FailoverPriLogicalIfName"] or ''
	else
		currentWan = "ALL"
	end
    for i = 1, 4 do
        if (currentWan ~= "" and currentWan ~= "ALL" and currentWan ~= "WAN" .. i) then
            configRow["ddns.ddnsService" .. i] = db.getAttribute("ddns", "interfaceName", "WAN" .. i, "ddnsService")
            configRow["ddns.hostname" .. i] = nil
            configRow["ddns.username" .. i] = nil
            configRow["ddns.password" .. i] = nil
            configRow["ddns.wildflag" .. i] = nil
            configRow["ddns.timePeriod" .. i] = nil
        end
    end
    errorFlag, statusCode = ddns.configWrap (configRow, "4")
    if (statusCode == "STATUS_OK") then 
        errorFlag = "OK"		   
        db.save()
    else
    	statusCode = "DDNS_CONFIG_FAILED"
    end

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage           	 
end

--
-- This routine reads in the current DDNS configuration information
-- into a configuration object.
function ddnsCfgInit(args)
    local configRow = {}
    for i = 1, 4 do
    	local intName = "WAN" .. i
        local ddnsRow = {}
    	ddnsRow = ddns.configGet (intName)
        if (ddnsRow["ddns.wildflag"] == nil or ddnsRow["ddns.wildflag"] == "") then
            ddnsRow["ddns.wildflag"] = "0"
        end
        if (ddnsRow["ddns.timePeriod"] == nil or ddnsRow["ddns.timePeriod"] == "") then
            ddnsRow["ddns.timePeriod"] = "0"
        end
        if (ddnsRow ~= nil) then
            configRow["ddns.ddnsService" .. i] = ddnsRow["ddns.ddnsService"]
            configRow["ddns.hostname" .. i] = ddnsRow["ddns.hostname"]
            configRow["ddns.username" .. i] = ddnsRow["ddns.username"]
            configRow["ddns.password" .. i] = ddnsRow["ddns.password"]
            configRow["ddns.wildflag" .. i] = ddnsRow["ddns.wildflag"]
            configRow["ddns.timePeriod" .. i] = ddnsRow["ddns.timePeriod"]
        end
    end
    return 1, configRow
end

--
-- This routine validates the DDNS configuration information.
function ddnsCfgInputVal(configRow)
    local currentWan = ""
    local wanModeRow = db.getRow("WanMode", "_ROWID_", "1")
	if (wanModeRow["WanMode.Wanmode"] == "0") then
		currentWan = wanModeRow["WanMode.DedicatedLogicalIfName"] or ''
	elseif (wanModeRow["WanMode.Wanmode"] == "2") then
		currentWan = wanModeRow["WanMode.FailoverPriLogicalIfName"] or ''
	else
		currentWan = "ALL"
	end
    for i = 1, 4 do
        if (currentWan ~= "" and currentWan ~= "ALL" and currentWan ~= "WAN" .. i) then
            configRow["ddns.ddnsService" .. i] = nil
        end
    end
    for i = 1, 4 do
        if (configRow["ddns.ddnsService" .. i] == "1" or configRow["ddns.ddnsService" .. i] == "2" or
            configRow["ddns.ddnsService" .. i] == "4" or configRow["ddns.ddnsService" .. i] == "5" ) then  
            if (configRow["ddns.username" .. i] == nil ) then
                printCLIError ("Invalid Username for WAN" .. i)
                return false
            end
            if (configRow["ddns.password" .. i] == nil ) then
                printCLIError ("Invalid Password for WAN" .. i)
                return false
            end
        end
        if (configRow["ddns.ddnsService" .. i] == "1" or configRow["ddns.ddnsService" .. i] == "2" or
            configRow["ddns.ddnsService" .. i] == "5" ) then  
            if (configRow["ddns.hostname" .. i] == nil ) then
                printCLIError ("Invalid Hostname for WAN" .. i)
                return false
            end
        end
    end
    return true
end

