--[[
#### Venu Babu .M
#### TeamF1
#### www.TeamF1.com
#### SEP 29, 2011

#### File: dhcpv6.lua
#### Description: dhcpv6 functions

#### Revisions:
]]--


--************* Requires *************

--************* Initial Code *************
--package dhcp
dhcpv6 = {}
--************* Functions ************

-- dhcpv6 config
function dhcpv6.config (tableName, inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(tableName, inputTable)
        elseif (operation == "edit") then
            return db.update(tableName, inputTable, rowid)
        elseif (operation == "delete") then
           for k,v in pairs(inputTable) do
                valid  = db.deleteRow(tableName, "_ROWID_", v)
                if (not valid) then  return false end
            end
        end
    end

    return false
end


function dhcpv6.import (dhcpv6Config)

   for i,v in ipairs (dhcpv6Config["client"]) do
        v = util.addPrefix (v, "dhcpv6c.");
        dhcpv6.config ("dhcpv6c", v, -1, "add")
   end
	
   for i,v in ipairs (dhcpv6Config["server"]) do
        v = util.addPrefix (v, "dhcpv6s.");
        dhcpv6.config ("dhcpv6s", v, -1, "add")
   end
	
   for i,v in ipairs (dhcpv6Config["pool"]) do
        v = util.addPrefix (v, "dhcpv6sLANAddrPool.");
        dhcpv6.config ("dhcpv6sLANAddrPool", v, -1, "add")
   end

end

function dhcpv6.export ()
	local dhcpv6Tbl = {}

 	local ctable = db.getTable("dhcpv6c" , false)
	if (ctable ~= nil) then
	    dhcpv6Tbl["client"] = ctable
	end

    local stbl = db.getTable("dhcpv6s" , false)
	if (stbl ~= nil) then
	    dhcpv6Tbl["server"] = stbl
	end

	local poolTbl = db.getTable("dhcpv6sLANAddrPool", false)
	if (poolTbl ~= nil) then
	    dhcpv6Tbl["pool"] =  poolTbl
	end

	return dhcpv6Tbl
end


if (config.register) then
   config.register("dhcpv6", dhcpv6.import, dhcpv6.export)
end

-- Updating Configurations
function dhcpv6.updateConfig (inputTable, rowid)
    -- updating DHCP Configurations
    local inTable = {}
    inTable["dhcpv6s.isEnabled"] = inputTable["dhcpv6s.isEnabled"]
    if (inputTable["dhcpv6s.isEnabled"] ~= 0) then
        inTable["dhcpv6s.domainName"] = inputTable["dhcpv6s.domainName"]
        inTable["dhcpv6s.serverPreference"] = inputTable["dhcpv6s.serverPreference"]
        inTable["dhcpv6s.useDNSServersFrom"] = inputTable["dhcpv6s.useDNSServersFrom"]
        inTable["dhcpv6s.primaryDNSServer"] = inputTable["dhcpv6s.primaryDNSServer"]
        inTable["dhcpv6s.secondaryDNSServer"] = inputTable["dhcpv6s.secondaryDNSServer"]
        inTable["dhcpv6s.leaseTime"] = inputTable["dhcpv6s.leaseTime"]
        inTable["dhcpv6s.statelessMode"] = inputTable["dhcpv6s.statelessMode"]
    end

    return db.update("dhcpv6s",inTable, rowid)

end


function dhcpv6.dhcpConfigWrap (inputTable, logIfName)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local where = "LogicalIfName = '" .. logIfName .."'"
    local whr = "_ROWID_ = '1'"
    local dhcpv6sConfig = db.getRowWhere ("dhcpv6s",where)
    local dhcpLdapServerConfig = db.getRowWhere ("DhcpLdapServerInfo",where)
    local rowid = nil
    --local addPoolRowId = nil

   -- Commenter below code, we have added addersspool as separate section in dmz_setup_ipv6 page
   -- as mentioned in the spr #27717

    --[[local dhcpv6sDMZAddrPool = {}
    
    dhcpv6sDMZAddrPool["dhcpv6sDMZAddrPool.startAddress"] = inputTable["dhcpv6sDMZAddrPool.startAddress"]
    dhcpv6sDMZAddrPool["dhcpv6sDMZAddrPool.endAddress"] = inputTable["dhcpv6sDMZAddrPool.endAddress"]
    dhcpv6sDMZAddrPool["dhcpv6sDMZAddrPool.prefixLength"] = inputTable["ifStatic.PrefixLength"]
    
    local addPool = db.getRowWhere ("dhcpv6sDMZAddrPool", whr)

    if (addPool ~= nil) then
        addPoolRowId = addPool["dhcpv6sDMZAddrPool._ROWID_"]
    end

    if(addPoolRowId ~= nil) then
        db.update("dhcpv6sDMZAddrPool", dhcpv6sDMZAddrPool, addPoolRowId)
    else
        db.insert("dhcpv6sDMZAddrPool", dhcpv6sDMZAddrPool)
    end]]--

    if (dhcpv6sConfig ~= nil) then
        rowid = dhcpv6sConfig["dhcpv6s._ROWID_"]
    end

    if (rowid ~= nil) then
         valid = dhcpv6.updateConfig (inputTable, rowid)
    end

    local dhcpLdapServerInTable = {}
    dhcpLdapServerInTable["DhcpLdapServerInfo.DhcpLdapStatus"] = inputTable["DhcpLdapServerInfo.DhcpLdapStatus"]
    if(dhcpLdapServerInTable["DhcpLdapServerInfo.DhcpLdapStatus"] ~= "0") then
        dhcpLdapServerInTable["DhcpLdapServerInfo.DhcpLdapServerIP"] = inputTable["DhcpLdapServerInfo.DhcpLdapServerIP"]
        dhcpLdapServerInTable["DhcpLdapServerInfo.DhcpLdapSearchBase"] = inputTable["DhcpLdapServerInfo.DhcpLdapSearchBase"]
        dhcpLdapServerInTable["DhcpLdapServerInfo.DhcpLdapServerPort"] = inputTable["DhcpLdapServerInfo.DhcpLdapServerPort"]
    end
    if (valid and dhcpLdapServerConfig ~= nil) then
        rowid = dhcpLdapServerConfig["DhcpLdapServerInfo._ROWID_"]
            if (valid) then
                valid = db.update("DhcpLdapServerInfo", dhcpLdapServerInTable, rowid)
            end
        end    
    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DHCP_CONFIG_FAILED"
    end
end

--dhcp6FixedIP config
function dhcpv6.fixedIpConfig (inputTable, rowid, operation, DBTable)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end

    if(operation == "add") then
    	if(db.existsRow(DBTable,"Ipv6Addr",inputTable["Dhcp6fixedIpAddress.Ipv6Addr"])) then
    		return "ERROR", "DUPLICATE_IPADDR"
    	end
    
    elseif(operation == "edit") then
    	local dhcp6FixedRowId = db.getAttribute("Dhcp6fixedIpAddress","Ipv6Addr",inputTable["Dhcp6fixedIpAddress.Ipv6Addr"],"_ROWID_")
    	if(dhcp6FixedRowId ~= nil and dhcp6FixedRowId ~= rowid) then
    		return "ERROR", "DUPLICATE_IPADDR"
    	end
    end

    db.beginTransaction() --begin transaction
    local valid = false
    if (operation == "add") then
        valid = db.insert("Dhcp6fixedIpAddress", inputTable)
    elseif (operation == "edit") then
	if(DBTable == "AvailableLanHost6") then
	   if(db.existsRow("Dhcp6fixedIpAddress","Ipv6Addr",inputTable["Dhcp6fixedIpAddress.Ipv6Addr"])) then
    		return "ERROR", "DUPLICATE_IPADDR"
    	end
    	if(db.existsRow("Dhcp6fixedIpAddress","MacAddr",inputTable["Dhcp6fixedIpAddress.MacAddr"])) then
    		return "ERROR", "DUPLICATE_MACADDR"
    	end
            valid = db.insert("Dhcp6fixedIpAddress", inputTable)
	else
            valid = db.update("Dhcp6fixedIpAddress", inputTable, rowid)
        end
    elseif (operation == "delete") then
        local i = 0
        for k,v in pairs(inputTable) do
            i = i + 1
            --local row = inputTable[i]
            valid = db.deleteRow("Dhcp6fixedIpAddress", "_ROWID_", v)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DHCPv6_CONFIG_FAILED"
    end
end

