function modeCheck()
    local ipv6Status = db.getAttribute ("networkInfo", "_ROWID_", 1, "netWorkMode")
    if (ipv6Status ~= "3") then
        printCLIError ("Please Set IP Mode to IPv4/IPv6");
        return false
    end
    return true
end


--
-- This routine implements adding the default ipv6 route on bdg interface for CLI
function tahiDefaultRouteAdd (ipAddr)
    if (modeCheck() == false) then 
        return
    end
    tahiDefaultRouteDel ()
    local x = os.execute ("route -A inet6 add default gw " .. ipAddr.. " dev bdg1")
end
--
-- This routine implements deleting the default ipv6 route for CLI
function tahiDefaultRouteDel ()
    if (modeCheck() == false) then 
        return
    end
    local x = os.execute ("ip -6 route del default")
end

-- This routine kills radvd for CLI
function tahiStopRA ()
    if (modeCheck() == false) then 
        return
    end
    local x = os.execute ("killall radvd 2>/dev/null")
end

-- This routine starts radvd with AdvRetransTimer(1000) for CLI
function tahiStartRA (fileName)
    if (modeCheck() == false) then 
        return
    end
    tahiStopRA ()
    local x = os.execute ("radvd -C " .. fileName)
end

-- This routine disables ipv6 for CLI
function tahiProcSet (param)
    if (modeCheck() == false) then 
        return
    end
    if (param == 0) then
        local x = os.execute ("echo \"1\">/proc/sys/net/ipv6/conf/bdg1/disable_ipv6")
        local y = os.execute ("ifconfig eth1 down")
    elseif (param == 1) then
        local x = os.execute ("echo \"0\">/proc/sys/net/ipv6/conf/bdg1/disable_ipv6")
        local y = os.execute ("ifconfig eth1 up")
    end

end

--This routine adds an ipv6 address on an interface for CLI
function tahiAliasAdd (ip6_address, interface)
    if (modeCheck() == false) then 
        return
    end
    local x =  os.execute ("ifconfig " .. interface .. " add " .. ip6_address.."/64");
end

--This routine deletes an ipv6 address on an interface for CLI
function tahiAliasDel (ip6_address, interface)
    if (modeCheck() == false) then 
        return
    end
    local x =  os.execute ("ifconfig  " .. interface .. " del " .. ip6_address.."/64");
end

--This routine deletes ipv6 neighbor cache for CLI
function tahiNeighCacheDel ()
    if (modeCheck() == false) then 
        return
    end
    local x =  os.execute ("ip -6 neigh flush all");
end

--This routine sets MTU on WAN for CLI
function tahiMtuSet (mtu)
    if (modeCheck() == false) then 
        return
    end
    local x =  os.execute ("ifconfig eth1 mtu " .. mtu);
end

function tahiStartRALanWanInit(args)
    local configRow = {}
    configRow["fileName"] = args[1]
    return 1, configRow
end
function tahiStartRALanWanSave (configRow)
    if (modeCheck() == false) then 
        return
    end
    if (configRow["LanPrefix"] == "" or configRow["LanPrefix"] == nil) then
        printCLIError ("Invalid Lan Prefix")
        return false
    end
    if (configRow["WanPrefix"] == "" or configRow["WanPrefix"] == nil) then
        printCLIError ("Invalid Wan Prefix")
        return false
    end
    if (configRow["LanPrefixLen"] == "" or configRow["LanPrefixLen"] == nil) then
        printCLIError ("Invalid Lan Prefix Length")
        return false
    end
    if (configRow["WanPrefixLen"] == "" or configRow["WanPrefixLen"] == nil) then
        printCLIError ("Invalid Wan Prefix Length")
        return false
    end
    tahiStopRA ()
    local f = io.open("".. configRow["fileName"],"w+")
    f:write("interface bdg1\n")
    f:write("{\n")
    f:write("\tAdvSendAdvert on;\n")
    f:write("\tMaxRtrAdvInterval 60;\n")
    f:write("\tMinRtrAdvInterval 45;\n")
    f:write("\tAdvDefaultLifetime 600;\n")
    f:write("\tprefix "..configRow["LanPrefix"].."/"..configRow["LanPrefixLen"].."\n")
    f:write("\t\t{\n")
    f:write("\t\tAdvOnLink on;\n")
    f:write("\t\tAdvAutonomous on;\n")
    f:write("\t\t};\n")
    f:write("\t};\n")
    f:write("interface eth1\n")
    f:write("{\n")
    f:write("\tAdvSendAdvert on;\n")
    f:write("\tMaxRtrAdvInterval 60;\n")
    f:write("\tMinRtrAdvInterval 45;\n")
    f:write("\tAdvDefaultLifetime 600;\n")
    f:write("\tprefix "..configRow["WanPrefix"].."/"..configRow["WanPrefixLen"].."\n")
    f:write("\t\t{\n")
    f:write("\t\tAdvOnLink on;\n")
    f:write("\t\tAdvAutonomous on;\n")
    f:write("\t\t};\n")
    f:write("\t};\n")
    f:close()
    local x = os.execute ("radvd -C " .. configRow["fileName"])
    local y = os.execute ("ip -6 addr add "..configRow["LanPrefix"].."1/"..configRow["LanPrefixLen"].." dev bdg1")
    local z = os.execute ("ip -6 addr add "..configRow["WanPrefix"].."1/"..configRow["WanPrefixLen"].." dev eth1")
end
