-- utilties.lua
-- Gaurav Mathur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 27dec11,akr fixed factory reset command's function for fvs318n
-- 09apr10,db modified USB related routines to support udev
-- 24feb10,db modified added support for cat,caopy,mount,unmount,md5sum
-- 23dec07,gnm modified utilCLIFactoryReset to prompt for user confirmation
-- 07dec07 gnm written
--
-- Description
-- CLI utility routines
require "teamf1lualib/config"
require "teamf1lualib/firmware"
--
-- This routine implements the ping command for the CLI
function utilCLIPing(ipToPing)
    local pingprog = db.getAttribute ("environment", "name", "PING_PROGRAM", 
	  "value")
    local pingfile = db.getAttribute("environment", "name", "PING_FILE_NAME", 
	  "value")
    local x = os.execute (pingprog .. " " .. ipToPing)
    printCLIError (x)
end
function utilCLIFirmwareUpgrade (filename, serverIpAddress)
    local FileName = filename
    local ServerIpAddress = serverIpAddress  
    local prepareImageDir = os.execute ("mkdir -p /var/image/ && rm -rf /var/image/* ")
    local TFTPCommand = "/usr/bin/tftp -g -l /var/image/" .. FileName .. " -r " .. FileName .. " " .. ServerIpAddress
    local upgradeProgram = "/pfrm2.0/bin/upgradeCustom.sh /var/image/" .. FileName
    --save reboot date_time
    local rebootTimeScript = db.getAttribute("environment","name","REBOOT_TIME_SAVE_SCRIPT","value")
    if ( rebootTimeScript ~= nil ) then os.execute(rebootTimeScript) end
    local uploadFirmwareStatus = os.execute (TFTPCommand)
    os.execute ("/pfrm2.0/bin/upgradeCopy.sh")
    os.execute("sleep 2")
    local upgradeFirmwareStatus = os.execute (upgradeProgram)
    return 0
end
-- function for config upload
function utilCLIConfigurationUpload (filename, serverIpAddress) 
	local FileName = filename
    	local ServerIpAddress = serverIpAddress
	local TFTPCommand = "/usr/bin/tftp -g -l /tmp/" .. FileName .. " -r " .. FileName .. " " .. ServerIpAddress
        --save reboot date_time
        local rebootTimeScript = db.getAttribute("environment","name","REBOOT_TIME_SAVE_SCRIPT","value")
        if ( rebootTimeScript ~= nil ) then os.execute(rebootTimeScript) end
	local uploadConfigurationStatus = os.execute(TFTPCommand)
	if ( config.verifyChecksum ("/tmp/" .. FileName) == "ok") then
		print("Uploading Configuration File ...\n")
		os.execute("rm -f /data/tmp/teamf1.cfg.ascii")
		os.execute("cp -f /tmp/" .. FileName .. " /data/tmp/teamf1.cfg.ascii")
		print("Now Router will REBOOT ...")
		os.execute("reboot")
		return true
	else
		print("Error in Checksum of config file !!!")
		return false
	end
end
-- function for config backup
function utilCLIConfigurationBackup (fileOnServer, serverIpAddress) 
    	local ServerIpAddress = serverIpAddress
	local FileName = fileOnServer
	-- goto /data/tmp/ and then execute the command to put the teamf1.cfg.ascii file to tftp server
	local TFTPCommand = "cd /data/tmp/ && tftp -p -l teamf1.cfg.ascii  -r " .. FileName .." " .. ServerIpAddress
	print("backing up Configuration File ...")
	local backupConfigurationStatus = os.execute(TFTPCommand)
	if ( backupConfigurationStatus == 256) then
		print("Error in Backup !!")
		return false
	else
		print("Backup DONE !!!")
		return true
	end
end

--
-- This routine implements the cat command for the CLI
function utilCLICat(fileName)
    local x = os.execute ("/bin/cat " .. fileName)
end

--
-- This routine implements the reboot command for the CLI
function utilCLIReboot(args)
    --save reboot date_time
    local rebootTimeScript = db.getAttribute("environment","name","REBOOT_TIME_SAVE_SCRIPT","value")
    if ( rebootTimeScript ~= nil ) then os.execute(rebootTimeScript) end
    db.setAttribute("reboot", "reboot._ROWID_", "1", "reboot", "1")
    local restartTime = db.getAttribute("environment", "name", "BOOTUP_TIME", "value")
    statusMessage = "Router will be up in " .. restartTime .. " seconds."
    printCLIError (statusMessage)	
end

--
-- This routine implements the md5sum command for the CLI
function utilCLIMd5sum(fileName)
    local x = os.execute ("/usr/bin/md5sum " .. fileName)
end

--
-- This routine implements the copy command for the CLI
function utilCLICopy(fileName, fileName1)
    local x = os.execute ("/bin/cp " .. fileName .. " " ..fileName1)
end

--function to ping through VPN Tunnel
function utilPingThroughVPNTunnel(ipToPing)
	local tempFile = "/tmp/_t"
	local pingVPNprog = db.getAttribute("environment", "name", "VPN_PING_PROGRAM", "value")	
	local lanConfig = db.getRowWhere ("ifStatic","LogicalIfName = 'LAN' and AddressFamily = 2")
	if (ipToPing ~= nil and pingVPNprog ~= nil and lanConfig ~= nil) then
		local VPNPingProg = pingVPNprog .. " " .. lanConfig["ifStatic.StaticIp"] .. " " .. ipToPing .. " " .. tempFile
		os.execute(VPNPingProg)
		os.execute("cat " .. tempFile)
		return true
	end
	return false
end
--function for traceroute 
function utilTraceroute(address)
	local traceRouteProg = db.getAttribute("environment", "name", "TRACEROUTE_PROGRAM", "value")	
	if ( traceRouteProg ~= nil and address ~= nil ) then
		local program= traceRouteProg .." " .. address
		os.execute(program)
		return true
	end
	return false
end

--function for DNS Lookup
function utilDNSLookup(name)
	local nsLookupProg = db.getAttribute("environment", "name", "NSLOOKUP_PROGRAM", "value")	
	if ( nsLookupProg ~= nil and name ~= nil ) then
		local prog = nsLookupProg .." " ..  name
		os.execute(prog)
		return true
	end
	return false
end

--function for displaying ipv6 routing table
function utilRoutingTableIPv6()
	local route6Prog = db.getAttribute("environment", "name", "ROUTE6_PROGRAM", "value")
	if ( route6Prog ~= nil ) then
		os.execute(route6Prog)
		return true
	end
	return false
end

--function for displaying ipv4 routing table
function utilRoutingTableIPv4()
	local routeProg = db.getAttribute("environment", "name", "ROUTE_PROGRAM", "value")
	if ( routeProg ~= nil ) then
		os.execute(routeProg.." -n")
		return true
	end
	return false
end

-- This routine implements the CLI command to restore factory default
-- settings on the system.
function utilCLIFactoryReset(args)
--    local triesMax = 5 -- hard-coded
    local triesCur = 0
    local confirm = ""
    local message = "Preparing to restore saved settings from user-provided " .. 
		  "file.\nWARNING: Current configuration will be erased.\n"

    printCLIError (message)
    while (confirm ~= "Y" and confirm ~= "N" ) do
	  printCLIError ("Are you sure you want to proceed [Y/N]?")
	  confirm = string.upper(io.stdin:read'*l')
--	  triesCur = triesCur + 1
    end

    if (confirm == "N" ) then return end
    	  	  
--[[    local fact_settingsfile = db.getAttribute("environment", "name", 
	  "TEAMF1_CFG_DEFAULTS_ASCII", "value")
    local SETTINGS_FILE = db.getAttribute("environment", "name", 
	  "TEAMF1_CFG_ASCII", "value")

    os.execute("rm -rf" .. " " .. SETTINGS_FILE)
    os.execute("cp" .. " " .. fact_settingsfile .. " " .. SETTINGS_FILE)

    -- copy file to flash
    local writeprog = db.getAttribute("environment", "name", "CFG_WRITE_PROGRAM",
	  	 "value")
    local configpart = db.getAttribute("environment", "name", 
	  "FLASH_CFG_PARTITION", "value")
    os.execute(writeprog .. " " .. configpart .. " " .. SETTINGS_FILE)

   local resetCommand = db.getAttribute("environment","name","FACTORY_RESET_SCRIPT","value")
    --save reboot date_time
    local rebootTimeScript = db.getAttribute("environment","name","REBOOT_TIME_SAVE_SCRIPT","value")
    if ( rebootTimeScript ~= nil ) then os.execute(rebootTimeScript) end
]]
   os.execute("/pfrm2.0/bin/factoryReset.sh")
end

--
-- This routine implements the firmwareVersion command for the CLI
function utilCLIFirmVer(args)
    local firmwareVer1 = ""
    local firmwareVer2 = ""
    status, firmwareVer1 = firmware.getVersion ("image1")
    status, firmwareVer2 = firmware.getVersion ("image2")
    if(db.existsRow("system","system._ROWID_","1")) then
	    sysName = db.getAttribute("system", "system._ROWID_", "1", "name")
    else
            sysName = db.getAttribute("environment", "name", "DEVICE_NAME", "value")
    end

    --local firmwareVer = db.getAttribute("system", "_ROWID_", "1", "firmwareVer")
    print ("Firmware Version : ".. firmwareVer1)
    print ("Secondary Firmware Version : ".. firmwareVer2)
end


--
--This routine is the helper rouine for utilCLIUsbTest
--
function utilCLIUsbTestHelp (ipAddr, fileName, folderName)
        
    x = os.execute ("tftp -g -r ".. fileName .." -l "..folderName..fileName.." ".. ipAddr)
    if ( x == 256) then
        print ("TFTP failed ")	
        return false
    end
    print ("Md5sum: ")
    x = os.execute ("/usr/bin/md5sum "..folderName..fileName)
    if ( x == 256) then
        print ("Md5Sum get Failed ")	
        return false
    end
    x = os.execute ("rm -rf "..folderName..fileName)
    if ( x == 256) then
        print ("File emoving Failed")	
        return false
    end
end


--
-- This routine implements the USB Testing command for the CLI
function utilCLIUsbTest(ipAddr, fileName)

    local modelName =  db.getAttribute("system", "_ROWID_", "1", "name")
    if (string.find (modelName,"DSR") == nil) then
        print ("This box doesn't belong to DSR family")
    end
    if (string.find (modelName,"1000") ~= nil) then
        file1 = io.open ("/var/usb1Info","r")
        file2 = io.open ("/var/usb2Info","r")
        if (file1 == nil or file2 == nil) then
            print ("Please Connect Devices to USB Ports")
            return false
        end
        fileContent1 = file1:read("*line")
        fileContent2 = file2:read("*line")
        if (string.find (fileContent1, "storage") == nil or string.find (fileContent2, "storage") == nil) then
            print ("Device connected is not storage device")
            return false
        end
        print ("Test for USB1:")
        utilCLIUsbTestHelp (ipAddr, fileName, "/usb1/")
        print ("Test for USB2:")
        utilCLIUsbTestHelp (ipAddr, fileName, "/usb2/")
    elseif (string.find (modelName,"500") ~= nil) then
        file1 = io.open ("/var/usb1Info","r")
        if (file1 == nil) then
            print ("Please Connect Device to USB Port")
            return false
        end
        fileContent1 = file1:read("*line")
        if (string.find (fileContent1, "storage") == nil) then
            print ("Device connected is not storage device")
            return false
        end
        print ("Test for USB1:")
        utilCLIUsbTestHelp (ipAddr, fileName, "/usb1/")
    else
        print ("This box doesn't belong to DSR-1000 or DSR-500 family")
    end
end

